@@ -14,6 +14,7 @@ from passlib.utils import sys_bits
 #local
 __all__ = [
     'custom_app_context',
+    'django_context',
     'ldap_context', 'ldap_nocrypt_context',
     'mysql_context', 'mysql4_context', 'mysql3_context',
     'phpass_context',
@@ -42,6 +43,18 @@ custom_app_context = LazyCryptContext(
     admin__sha256_crypt__default_rounds = 80000,
     )
 
+#=========================================================
+#django
+#=========================================================
+django_context = LazyCryptContext(
+    schemes=[
+        "django_salted_sha1", "django_salted_md5", "django_des_crypt",
+        "hex_md5", "django_disabled",
+    ],
+    default="django_salted_sha1",
+    deprecated=["hex_md5"],
+)
+
 #=========================================================
 #ldap
 #=========================================================
@@ -85,8 +98,10 @@ postgres_context = LazyCryptContext(["postgres_md5"])
 #phpass & variants
 #=========================================================
 phpass_context = LazyCryptContext(
-    schemes=["bcrypt", "bsdi_crypt", "phpass",],
-    default="bcrypt" if hash.bcrypt.has_backend() else "bsdi_crypt",
+    schemes=["bcrypt", "phpass", "bsdi_crypt", ],
+
+    #XXX: make this lazy as well? causes bcrypt hash to always get loaded
+    default="bcrypt" if hash.bcrypt.has_backend() else "phpass",
     )
 
 phpbb3_context = LazyCryptContext(["phpass"], phpass__ident="H")
