@@ -3,9 +3,19 @@
 #imports
 #=========================================================
 from __future__ import with_statement
+from passlib.utils import py32_lang
 #core
 from cStringIO import StringIO
-from ConfigParser import ConfigParser
+# Py2k #
+    #note: importing ConfigParser to handle passlib 1.4 / earlier files
+from ConfigParser import SafeConfigParser,ConfigParser,InterpolationSyntaxError
+# Py3k #
+#if py32_lang:
+#    #Py3.2 removed old ConfigParser, put SafeConfigParser in it's place
+#    from ConfigParser import ConfigParser as SafeConfigParser
+#else:
+#    from ConfigParser import SafeConfigParser
+# end Py3k #
 import inspect
 import re
 import hashlib
@@ -18,7 +28,8 @@ from warnings import warn
 from pkg_resources import resource_string
 #libs
 from passlib.registry import get_crypt_handler, _unload_handler_name
-from passlib.utils import Undef, is_crypt_handler, splitcomma, rng
+from passlib.utils import to_bytes, to_unicode, bytes, Undef, \
+                          is_crypt_handler, splitcomma, rng
 #pkg
 #local
 __all__ = [
@@ -29,6 +40,28 @@ __all__ = [
 #=========================================================
 #crypt policy
 #=========================================================
+
+#--------------------------------------------------------
+#constants controlling parsing of special kwds
+#--------------------------------------------------------
+
+#: CryptContext kwds which aren't allowed to have category specifiers
+_forbidden_category_context_options = frozenset([ "schemes", ])
+    #NOTE: forbidding 'schemes' because it would really complicate the behavior
+    # of CryptContext.identify & CryptContext.lookup.
+    # most useful behaviors here can be had by overriding deprecated
+    # and default, anyways.
+
+#: hash settings which aren't allowed to be set via policy
+_forbidden_hash_options = frozenset([ "salt" ])
+    #NOTE: doing this for security purposes, why would you ever want a fixed salt?
+
+#: CryptContext kwds which should be parsed into comma separated list of strings
+_context_comma_options = frozenset([ "schemes", "deprecated" ])
+
+#--------------------------------------------------------
+#parsing helpers
+#--------------------------------------------------------
 def _parse_policy_key(key):
     "helper to normalize & parse policy keys; returns ``(category, name, option)``"
     orig = key
@@ -54,9 +87,9 @@ def _parse_policy_key(key):
 
 def _parse_policy_value(cat, name, opt, value):
     "helper to parse policy values"
-    #FIXME: kinda primitive :|
+    #FIXME: kinda primitive to parse things this way :|
     if name == "context":
-        if opt == "schemes" or opt == "deprecated":
+        if opt in _context_comma_options:
             if isinstance(value, str):
                 return splitcomma(value)
         elif opt == "min_verify_time":
@@ -71,23 +104,52 @@ def _parse_policy_value(cat, name, opt, value):
 
 def parse_policy_items(source):
     "helper to parse CryptPolicy options"
+    # py2k #
     if hasattr(source, "iteritems"):
         source = source.iteritems()
+    # py3k #
+    #if hasattr(source, "items"):
+    #    source = source.items()
+    # end py3k #
     for key, value in source:
         cat, name, opt = _parse_policy_key(key)
         if name == "context":
-            if cat and opt == "schemes":
-                raise KeyError("current code does not support per-category schemes")
-                #NOTE: forbidding this because it would really complicate the behavior
-                # of CryptContext.identify & CryptContext.lookup.
-                # most useful behaviors here can be had by overridding deprecated and default, anyways.
+            if cat and opt in _forbidden_category_context_options:
+                raise KeyError("%r context option is not allowed per-category" % (opt,))
         else:
-            if opt == "salt":
-                raise KeyError("'salt' option is not allowed to be set via a policy object")
-                #NOTE: doing this for security purposes, why would you ever want a fixed salt?
+            if opt in _forbidden_hash_options:
+                raise KeyError("%r handler option is not allowed to be set via a policy object" % (opt,))
         value = _parse_policy_value(cat, name, opt, value)
         yield cat, name, opt, value
 
+# Py2k #
+def _is_legacy_parse_error(err):
+    "helper for parsing config files"
+    #NOTE: passlib 1.4 and earlier used ConfigParser,
+    # when they should have been using SafeConfigParser
+    # (which passlib 1.5+ switched to)
+    # this has no real security effects re: passlib,
+    # but some 1.4 config files that have "vary_rounds = 10%"
+    # may throw an error under SafeConfigParser,
+    # and should read "vary_rounds = 10%%"
+    #
+    # passlib 1.6 and on will only use SafeConfigParser,
+    # but passlib 1.5 tries to detect the above 10% error,
+    # issue a warning, and retry w/ ConfigParser,
+    # for backward compat.
+    #
+    # this function's purpose is to encapsulate that
+    # backward-compat behavior.
+    value = err.args[0]
+    #'%' must be followed by '%' or '(', found: '%'
+    if value == "'%' must be followed by '%' or '(', found: '%'":
+        return True
+    return False
+# end Py2k #
+
+#--------------------------------------------------------
+#policy class proper
+#--------------------------------------------------------
 class CryptPolicy(object):
     """stores configuration options for a CryptContext object.
 
@@ -125,40 +187,103 @@ class CryptPolicy(object):
 
     .. note::
         Instances of CryptPolicy should be treated as immutable.
+        Use the :meth:`replace` method to mutate existing instances.
     """
 
     #=========================================================
     #class methods
     #=========================================================
     @classmethod
-    def from_path(cls, path, section="passlib"):
+    def from_path(cls, path, section="passlib", encoding="utf-8"):
         """create new policy from specified section of an ini file.
 
         :arg path: path to ini file
         :param section: option name of section to read from.
+        :arg encoding: optional encoding (defaults to utf-8)
 
         :raises EnvironmentError: if the file cannot be read
 
         :returns: new CryptPolicy instance.
         """
-        p = ConfigParser()
-        if not p.read([path]):
-            raise EnvironmentError("failed to read config file")
-        return cls(**dict(p.items(section)))
+        #NOTE: we want config parser object to have native strings as keys.
+        #      so we parse as bytes under py2, and unicode under py3.
+        #
+        #      encoding issues are handled under py2 via to_bytes(),
+        #      which ensures everything is utf-8 internally.
+
+        # Py2k #
+        if encoding == "utf-8":
+            #we want utf-8 anyways, so just load file in raw mode.
+            with open(path, "rb") as stream:
+                return cls._from_stream(stream, section, path)
+        else:
+            #kinda hacked - load whole file, transcode, and parse.
+            with open(path, "rb") as stream:
+                source = stream.read()
+            source = source.decode(encoding).encode("utf-8")
+            return cls._from_stream(StringIO(source), section, path)
+        # Py3k #
+        #with open(path, "r", encoding=encoding) as stream:
+        #    return cls._from_stream(stream, section, path)
+        # end Py3k #
 
     @classmethod
-    def from_string(cls, source, section="passlib"):
+    def from_string(cls, source, section="passlib", encoding="utf-8"):
         """create new policy from specified section of an ini-formatted string.
 
-        :arg source: string containing ini-formatted content.
+        :arg source: bytes/unicode string containing ini-formatted content.
         :param section: option name of section to read from.
+        :arg encoding: optional encoding if source is bytes (defaults to utf-8)
 
         :returns: new CryptPolicy instance.
         """
-        p = ConfigParser()
-        b = StringIO(source)
-        p.readfp(b)
-        return cls(**dict(p.items(section)))
+        #NOTE: we want config parser object to have native strings as keys.
+        #      so we parse as bytes under py2, and unicode under py3.
+        #      to handle encoding issues under py2, we use
+        #      "to_bytes()" to transcode to utf-8 as needed.
+
+        # Py2k #
+        source = to_bytes(source, "utf-8", source_encoding=encoding, errname="source")
+        # Py3k #
+        #source = to_unicode(source, encoding, errname="source")
+        # end Py3k #
+        return cls._from_stream(StringIO(source), section, "<???>")
+
+    @classmethod
+    def _from_stream(cls, stream, section, filename=None):
+        "helper for from_string / from_path"
+        # Py2k #
+        pos = stream.tell()
+        # end Py2k #
+
+        p = SafeConfigParser()
+        if py32_lang:
+            # Py3.2 deprecated readfp
+            p.read_file(stream, filename or "<???>")
+        else:
+            p.readfp(stream, filename or "<???>")
+
+        # Py2k #
+        try:
+            items = p.items(section)
+        except InterpolationSyntaxError, err:
+            if not _is_legacy_parse_error(err):
+                raise
+            #support for deprecated 1.4 behavior, will be removed in 1.6
+            if filename:
+                warn("from_path(): the file %r contains an unescaped '%%', this will be fatal in passlib 1.6" % (filename,), stacklevel=3)
+            else:
+                warn("from_string(): the provided string contains an unescaped '%', this will be fatal in passlib 1.6", stacklevel=3)
+            p = ConfigParser()
+            stream.seek(pos)
+            p.readfp(stream)
+            items = p.items(section)
+
+        # py3k #
+        #items = p.items(section)
+        # end py3k #
+
+        return cls(**dict(items))
 
     @classmethod
     def from_source(cls, source):
@@ -181,7 +306,7 @@ class CryptPolicy(object):
         elif isinstance(source, dict):
             return cls(**source)
 
-        elif isinstance(source, (str,unicode)):
+        elif isinstance(source, (bytes,unicode)):
             #FIXME: this autodetection makes me uncomfortable...
             if any(c in source for c in "\n\r\t") or not source.strip(" \t./\;:"): #none of these chars should be in filepaths, but should be in config string
                 return cls.from_string(source)
@@ -226,13 +351,12 @@ class CryptPolicy(object):
         """return copy of policy, with specified options replaced by new values.
 
         this is essentially a convience wrapper around :meth:`from_sources`,
-        except that it always inserts the current policy as the first element
-        in the list.
-
+        except that it always inserts the current policy
+        as the first element in the list;
         this allows easily making minor changes from an existing policy object.
 
-        :param args: optional list of sources as accepted by :meth:`from_sources`.
-        :param kwds: optional specific options to override in the new policy.
+        :param \*args: optional list of sources as accepted by :meth:`from_sources`.
+        :param \*\*kwds: optional specific options to override in the new policy.
 
         :returns: new CryptPolicy instance
         """
@@ -283,7 +407,8 @@ class CryptPolicy(object):
         #
         #init cache & options
         #
-        options = self._options = {None:{"context":{}}}
+        context_options = {}
+        options = self._options = {None:{"context":context_options}}
         self._cache = {}
 
         #
@@ -302,9 +427,9 @@ class CryptPolicy(object):
         #
         #parse list of schemes, and resolve to handlers.
         #
+        schemes = context_options.get("schemes") or []
         handlers = self._handlers = []
-        seen = set()
-        schemes = options[None]['context'].get("schemes") or []
+        handler_names = set()
         for scheme in schemes:
             #resolve & validate handler
             if is_crypt_handler(scheme):
@@ -316,12 +441,13 @@ class CryptPolicy(object):
                 raise TypeError("handler lacks name: %r" % (handler,))
 
             #check name hasn't been re-used
-            if name in seen:
+            if name in handler_names:
+                #XXX: should this just be a warning ?
                 raise KeyError("multiple handlers with same name: %r" % (name,))
-            seen.add(name)
 
             #add to handler list
             handlers.append(handler)
+            handler_names.add(name)
 
         #
         #build _deprecated & _default maps
@@ -335,11 +461,11 @@ class CryptPolicy(object):
                 continue
 
             #list of deprecated schemes
-            deps = kwds.get("deprecated")
+            deps = kwds.get("deprecated") or []
             if deps:
                 if handlers:
                     for scheme in deps:
-                        if scheme not in seen:
+                        if scheme not in handler_names:
                             raise KeyError("known scheme in deprecated list: %r" % (scheme,))
                 dmap[cat] = frozenset(deps)
 
@@ -349,7 +475,7 @@ class CryptPolicy(object):
                 if handlers:
                     if hasattr(fb, "name"):
                         fb = fb.name
-                    if fb not in seen:
+                    if fb not in handler_names:
                         raise KeyError("unknown scheme set as default: %r" % (fb,))
                     fmap[cat] = self.get_handler(fb, required=True)
                 else:
@@ -560,23 +686,37 @@ class CryptPolicy(object):
         "return policy as dictionary of keywords"
         return dict(self.iter_config(resolve=resolve))
 
+    def _escape_ini_pair(self, k, v):
+        if isinstance(v, str):
+            v = v.replace("%", "%%") #escape any percent signs.
+        elif isinstance(v, (int, long)):
+            v = str(v)
+        return k,v
+
     def _write_to_parser(self, parser, section):
         "helper for to_string / to_file"
         parser.add_section(section)
         for k,v in self.iter_config(ini=True):
+            k,v = self._escape_ini_pair(k,v)
             parser.set(section, k,v)
 
     def to_file(self, stream, section="passlib"):
         "serialize to INI format and write to specified stream"
-        p = ConfigParser()
+        p = SafeConfigParser()
         self._write_to_parser(p, section)
         p.write(stream)
 
-    def to_string(self, section="passlib"):
+    def to_string(self, section="passlib", encoding=None):
         "render to INI string; inverse of from_string() constructor"
-        b = StringIO()
-        self.to_file(b, section)
-        return b.getvalue()
+        buf = StringIO()
+        self.to_file(buf, section)
+        out = buf.getvalue()
+        # Py2k #
+        out = out.decode("utf-8")
+        # end Py2k #
+        if encoding:
+            out = out.encode(encoding)
+        return out
 
     ##def to_path(self, path, section="passlib", update=False):
     ##    "write to INI file"
@@ -697,6 +837,9 @@ class CryptContext(object):
                     if isinstance(vr, str):
                         rc = getattr(handler, "rounds_cost", "linear")
                         vr = int(vr.rstrip("%"))
+                            #NOTE: deliberately strip >1 %,
+                            #in case an interpolation-escaped %%
+                            #makes it through to here.
                         assert 0 <= vr < 100
                         if rc == "log2":
                             #let % variance scale the number of actual rounds, not the logarithmic value
