@@ -59,7 +59,7 @@ class PasslibRegistryProxy(object):
     #eoc
     #=========================================================
 
-#singleton instance
+#singleton instance - available publicallly as 'passlib.hash'
 _proxy = PasslibRegistryProxy()
 
 #==========================================================
@@ -79,9 +79,16 @@ _handler_locations = {
     "bcrypt":           ("passlib.handlers.bcrypt",      "bcrypt"),
     "bigcrypt":         ("passlib.handlers.des_crypt",   "bigcrypt"),
     "bsdi_crypt":       ("passlib.handlers.des_crypt",   "bsdi_crypt"),
+    "cta_pbkdf2_sha1":  ("passlib.handlers.pbkdf2",      "cta_pbkdf2_sha1"),
     "crypt16":          ("passlib.handlers.des_crypt",   "crypt16"),
     "des_crypt":        ("passlib.handlers.des_crypt",   "des_crypt"),
+    "django_salted_sha1":
+                        ("passlib.handlers.django",      "django_salted_sha1"),
+    "django_salted_md5":("passlib.handlers.django",      "django_salted_md5"),
+    "django_des_crypt": ("passlib.handlers.django",      "django_des_crypt"),
+    "django_disabled":  ("passlib.handlers.django",      "django_disabled"),
     "dlitz_pbkdf2_sha1":("passlib.handlers.pbkdf2",      "dlitz_pbkdf2_sha1"),
+    "fshp":             ("passlib.handlers.fshp",        "fshp"),
     "grub_pbkdf2_sha512":
                         ("passlib.handlers.pbkdf2",      "grub_pbkdf2_sha512"),
     "hex_md4":          ("passlib.handlers.digests",     "hex_md4"),
@@ -149,8 +156,19 @@ def register_crypt_handler_path(name, path):
     :arg path: module import path
 
     the specified module path should contain a password hash handler
-    called :samp:`{name}`, or the path may contain a semicolon,
+    called :samp:`{name}`, or the path may contain a colon,
     specifying the module and module attribute to use.
+    for example, the following would cause ``get_handler("myhash")`` to look
+    for a class named ``myhash`` within the ``myapp.helpers`` module::
+
+        >>> from passlib.registry import registry_crypt_handler_path
+        >>> registry_crypt_handler_path("myhash", "myapp.helpers")
+
+    ...while this form would cause ``get_handler("myhash")`` to look
+    for a class name ``MyHash`` within the ``myapp.helpers`` module::
+
+        >>> from passlib.registry import registry_crypt_handler_path
+        >>> registry_crypt_handler_path("myhash", "myapp.helpers:MyHash")
     """
     global _handler_locations
     if ':' in path:
@@ -167,6 +185,9 @@ def register_crypt_handler(handler, force=False, name=None):
 
     :arg handler: the password hash handler to register
     :param force: force override of existing handler (defaults to False)
+    :param name:
+        [internal kwd] if specified, ensures ``handler.name``
+        matches this value, or raises :exc:`ValueError`.
 
     :raises TypeError:
         if the specified object does not appear to be a valid handler.
@@ -228,23 +249,23 @@ def get_crypt_handler(name, default=Undef):
     it checks if the location is known, and loads it first.
 
     :arg name: name of handler to return
-    :param default: if specified, returns default value if no handler found.
+    :param default: optional default value to return if no handler with specified name is found.
 
-    :raises KeyError: if no handler matching that name is found, and no default specified
+    :raises KeyError: if no handler matching that name is found, and no default specified, a KeyError will be raised.
 
-    :returns: handler attached to name, or default if specified
+    :returns: handler attached to name, or default value (if specified).
     """
     global _handlers, _handler_locations
 
     #check if handler loaded
-    handler = _handlers.get(name, None)
+    handler = _handlers.get(name)
     if handler:
         return handler
 
     #normalize name (and if changed, check dict again)
     alt = name.replace("-","_").lower()
     if alt != name:
-        warn("handler names be lower-case, and use underscores instead of hyphens: %r => %r" % (name, alt))
+        warn("handler names should be lower-case, and use underscores instead of hyphens: %r => %r" % (name, alt))
         name = alt
 
         #check if handler loaded
