@@ -3,10 +3,10 @@
 #imports
 #=================================================================================
 #core
-from cStringIO import StringIO
 import logging; log = logging.getLogger(__name__)
 #site
 #pkg
+from passlib.utils import bytes, bjoin, bchrs, bord, belem_join
 #local
 __all__ = [
     "CHARS",
@@ -25,58 +25,63 @@ __all__ = [
 #=================================================================================
 #6 bit value <-> char mapping, and other internal helpers
 #=================================================================================
-CHARS = "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
 
-#base64 char sequence
-encode_6bit = CHARS.__getitem__ # int -> char
+#: hash64 char sequence
+CHARS = u"./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
+BCHARS = CHARS.encode("ascii")
 
-#inverse map (char->value)
-_CHARIDX = dict( (c,i) for i,c in enumerate(CHARS))
-decode_6bit = _CHARIDX.__getitem__ # char -> int
+#: encode int -> hash64 char as efficiently as possible, w/ minimal checking
+# Py2k #
+_encode_6bit = BCHARS.__getitem__
+# Py3k #
+#_encode_6bit = lambda v: BCHARS[v:v+1]
+# end Py3k #
 
-_sjoin = "".join
 
-try:
-    _bjoin = bytes().join
-except NameError: #pragma: no cover - though must test under py25
-    _bjoin = _sjoin
+#: decode hash64 char -> int as efficiently as possible, w/ minimal checking
+_CHARIDX = dict((_encode_6bit(i),i) for i in xrange(64))
+_decode_6bit = _CHARIDX.__getitem__ # char -> int
+
+#for py3, enhance _CHARIDX to also support int value of bytes
+# Py3k #
+#_CHARIDX.update((v,i) for i,v in enumerate(BCHARS))
+# end Py3k #
 
 #=================================================================================
 #encode offsets from buffer - used by md5_crypt, sha_crypt, et al
 #=================================================================================
 
-def encode_bytes(source):
-    "encode byte string to h64 format"
+def _encode_bytes_helper(source):
     #FIXME: do something much more efficient here.
     # can't quite just use base64 and then translate chars,
     # since this scheme is little-endian.
-    out = StringIO()
-    write = out.write
     end = len(source)
     tail = end % 3
     end -= tail
     idx = 0
     while idx < end:
-        v1 = ord(source[idx])
-        v2 = ord(source[idx+1])
-        v3 = ord(source[idx+2])
-        write(encode_int24(v1 + (v2<<8) + (v3<<16)))
+        v1 = bord(source[idx])
+        v2 = bord(source[idx+1])
+        v3 = bord(source[idx+2])
+        yield encode_int24(v1 + (v2<<8) + (v3<<16))
         idx += 3
     if tail:
-        v1 = ord(source[idx])
+        v1 = bord(source[idx])
         if tail == 1:
             #NOTE: 4 msb of int are always 0
-            write(encode_int12(v1))
+            yield encode_int12(v1)
         else:
             #NOTE: 2 msb of int are always 0
-            v2 = ord(source[idx+1])
-            write(encode_int18(v1 + (v2<<8)))
-    return out.getvalue()
+            v2 = bord(source[idx+1])
+            yield encode_int18(v1 + (v2<<8))
 
-def decode_bytes(source):
-    "decode h64 format into byte string"
-    out = StringIO()
-    write = out.write
+def encode_bytes(source):
+    "encode byte string to h64 format"
+    if not isinstance(source, bytes):
+        raise TypeError("source must be bytes, not %s" % (type(source),))
+    return bjoin(_encode_bytes_helper(source))
+
+def _decode_bytes_helper(source):
     end = len(source)
     tail = end % 4
     if tail == 1:
@@ -86,24 +91,31 @@ def decode_bytes(source):
     idx = 0
     while idx < end:
         v = decode_int24(source[idx:idx+4])
-        write(chr(v&0xff) + chr((v>>8)&0xff) + chr(v>>16))
+        yield bchrs(v&0xff, (v>>8)&0xff, v>>16)
         idx += 4
     if tail:
         if tail == 2:
             #NOTE: 2 msb of int are ignored (should be 0)
             v = decode_int12(source[idx:idx+2])
-            write(chr(v&0xff))
+            yield bchrs(v&0xff)
         else:
             #NOTE: 4 msb of int are ignored (should be 0)
             v = decode_int18(source[idx:idx+3])
-            write(chr(v&0xff) + chr((v>>8)&0xff))
-    return out.getvalue()
+            yield bchrs(v&0xff, (v>>8)&0xff)
+
+def decode_bytes(source):
+    "decode h64 format into byte string"
+    if not isinstance(source, bytes):
+        raise TypeError("source must be bytes, not %s" % (type(source),))
+    return bjoin(_decode_bytes_helper(source))
 
 def encode_transposed_bytes(source, offsets):
     "encode byte string to h64 format, using offset list to transpose elements"
+    if not isinstance(source, bytes):
+        raise TypeError("source must be bytes, not %s" % (type(source),))
     #XXX: could make this a dup of encode_bytes(), which directly accesses source[offsets[idx]],
     # but speed isn't *that* critical for this function
-    tmp = _bjoin(source[off] for off in offsets)
+    tmp = belem_join(source[off] for off in offsets)
     return encode_bytes(tmp)
 
 def decode_transposed_bytes(source, offsets):
@@ -113,16 +125,18 @@ def decode_transposed_bytes(source, offsets):
     buf = [None] * len(offsets)
     for off, char in zip(offsets, tmp):
         buf[off] = char
-    return _bjoin(buf)
+    return belem_join(buf)
 
 #=================================================================================
 # int <-> b64 string, used by des_crypt, ext_des_crypt
 #=================================================================================
 
-def decode_int6(value):
+def decode_int6(source):
     "decodes single hash64 character -> 6-bit integer"
+    if not isinstance(source, bytes):
+        raise TypeError("source must be bytes, not %s" % (type(source),))
     try:
-        return decode_6bit(value)
+        return _decode_6bit(source)
     except KeyError:
         raise ValueError("invalid character")
 
@@ -130,80 +144,86 @@ def encode_int6(value):
     "encodes 6-bit integer -> single hash64 character"
     if value < 0 or value > 63:
         raise ValueError("value out of range")
-    return encode_6bit(value)
+    return _encode_6bit(value)
 
 #---------------------------------------------------------------------
 
-def decode_int12(value):
+def decode_int12(source):
     "decodes 2 char hash64 string -> 12-bit integer (little-endian order)"
     #NOTE: this is optimized form of decode_int(value) for 4 chars
+    if not isinstance(source, bytes):
+        raise TypeError("source must be bytes, not %s" % (type(source),))
     try:
-        return (decode_6bit(value[1])<<6)+decode_6bit(value[0])
+        return (_decode_6bit(source[1])<<6)+_decode_6bit(source[0])
     except KeyError:
         raise ValueError("invalid character")
 
 def encode_int12(value):
     "encodes 12-bit integer -> 2 char hash64 string (little-endian order)"
     #NOTE: this is optimized form of encode_int(value,2)
-    return  encode_6bit(value & 0x3f) + encode_6bit((value>>6) & 0x3f)
+    return  _encode_6bit(value & 0x3f) + _encode_6bit((value>>6) & 0x3f)
 
 #---------------------------------------------------------------------
-def decode_int18(value):
+def decode_int18(source):
     "decodes 3 char hash64 string -> 18-bit integer (little-endian order)"
     #NOTE: this is optimized form of decode_int(value) for 3 chars
+    if not isinstance(source, bytes):
+        raise TypeError("source must be bytes, not %s" % (type(source),))
     return (
-        decode_6bit(value[0]) +
-        (decode_6bit(value[1])<<6) +
-        (decode_6bit(value[2])<<12)
+        _decode_6bit(source[0]) +
+        (_decode_6bit(source[1])<<6) +
+        (_decode_6bit(source[2])<<12)
         )
 
 def encode_int18(value):
     "encodes 18-bit integer -> 3 char hash64 string (little-endian order)"
     #NOTE: this is optimized form of encode_int(value,3)
     return (
-        encode_6bit(value & 0x3f) +
-        encode_6bit((value>>6) & 0x3f) +
-        encode_6bit((value>>12) & 0x3f)
+        _encode_6bit(value & 0x3f) +
+        _encode_6bit((value>>6) & 0x3f) +
+        _encode_6bit((value>>12) & 0x3f)
         )
 
 #---------------------------------------------------------------------
 
-def decode_int24(value):
+def decode_int24(source):
     "decodes 4 char hash64 string -> 24-bit integer (little-endian order)"
-    #NOTE: this is optimized form of decode_int(value) for 4 chars
+    #NOTE: this is optimized form of decode_int(source) for 4 chars
+    if not isinstance(source, bytes):
+        raise TypeError("source must be bytes, not %s" % (type(source),))
     try:
-        return  decode_6bit(value[0]) +\
-                (decode_6bit(value[1])<<6)+\
-                (decode_6bit(value[2])<<12)+\
-                (decode_6bit(value[3])<<18)
+        return  _decode_6bit(source[0]) +\
+                (_decode_6bit(source[1])<<6)+\
+                (_decode_6bit(source[2])<<12)+\
+                (_decode_6bit(source[3])<<18)
     except KeyError:
         raise ValueError("invalid character")
 
 def encode_int24(value):
     "encodes 24-bit integer -> 4 char hash64 string (little-endian order)"
     #NOTE: this is optimized form of encode_int(value,4)
-    return  encode_6bit(value & 0x3f) + \
-            encode_6bit((value>>6) & 0x3f) + \
-            encode_6bit((value>>12) & 0x3f) + \
-            encode_6bit((value>>18) & 0x3f)
+    return  _encode_6bit(value & 0x3f) + \
+            _encode_6bit((value>>6) & 0x3f) + \
+            _encode_6bit((value>>12) & 0x3f) + \
+            _encode_6bit((value>>18) & 0x3f)
 
 #---------------------------------------------------------------------
 
-def decode_int64(value):
+def decode_int64(source):
     "decodes 11 char hash64 string -> 64-bit integer (little-endian order; 2 msb assumed to be padding)"
-    return decode_int(value)
+    return decode_int(source)
 
 def encode_int64(value):
     "encodes 64-bit integer -> 11 char hash64 string (little-endian order; 2 msb of 0's added as padding)"
     return encode_int(value, 11)
 
-def decode_dc_int64(value):
+def decode_dc_int64(source):
     """decode 11 char hash64 string -> 64-bit integer (big-endian order; 2 lsb assumed to be padding)
 
     this format is used primarily by des-crypt & variants to encode the DES output value
     used as a checksum.
     """
-    return decode_int(value, True)>>2
+    return decode_int(source, True)>>2
 
 def encode_dc_int64(value):
     """encode 64-bit integer -> 11 char hash64 string (big-endian order; 2 lsb added as padding)
@@ -227,6 +247,8 @@ def decode_int(source, big=False):
     :returns:
         a integer whose value is in ``range(0,2**(6*len(source)))``
     """
+    if not isinstance(source, bytes):
+        raise TypeError("source must be bytes, not %s" % (type(source),))
     #FORMAT: little-endian, each char contributes 6 bits,
     # char value = index in H64_CHARS string
     if not big:
@@ -234,7 +256,8 @@ def decode_int(source, big=False):
     try:
         out = 0
         for c in source:
-                out = (out<<6) + decode_6bit(c)
+            #NOTE: under py3, 'c' is int, relying on _CHARIDX to support this.
+            out = (out<<6) + _decode_6bit(c)
         return out
     except KeyError:
         raise ValueError("invalid character in string")
@@ -255,8 +278,8 @@ def encode_int(value, count, big=False):
         itr = xrange(6*count-6, -6, -6)
     else:
         itr = xrange(0, 6*count, 6)
-    return _sjoin(
-        encode_6bit((value>>off) & 0x3f)
+    return bjoin(
+        _encode_6bit((value>>off) & 0x3f)
         for off in itr
     )
 
